--- 模块功能：看门狗功能
-- @author openLuat
-- @module uartPluse
-- @license MIT
-- @copyright openLuat
-- @release 2018.03.27

module(...,package.seeall)

require"utils"
require"pm"

--串口ID
local UART_ID = _G.moduleType == 2 and 2 or 1
--喂狗数据
local feed = string.fromHex("03000000")
--串口读到的数据缓冲区
local rdbuf = ""
--待发送消息队列
local msgQuene = {}
--插入消息
local function insert(data,delay)
    table.insert(msgQuene, {d=data,t=delay})
    sys.publish("UART_INSERT")
end

--检验函数
local function checksum(s)
    local ret = 0
    for i=1,s:len() do
        ret = bit.bxor(ret,s:byte(i))
    end
    return string.char(ret)
end

--发送串口信息线程
sys.taskInit(function ()
    sys.wait(10000)
    insert(feed)
    while true do
        while #msgQuene>0 do
            pm.wake("uartPluse")
            local outMsg = table.remove(msgQuene,1)
            write(outMsg.d..checksum(outMsg.d:sub(2)))
            if outMsg.t ~= nil then--如果有等待时间
                log.info("uart wait",outMsg.t)
                sys.wait(outMsg.t)
            end
        end
        sys.wait(200)
        pm.sleep("uartPluse")
        sys.waitUntil("UART_INSERT")
    end
end)

--发送串口信息
function sendPluse(high,low,num)
    function cs(sum)--防止大于255
        return sum > 255 and 255 or sum
    end
    high = cs(high)
    low = cs(low)
    num = cs(num)
    local delay = (high+low)*num--+2000
    msgQuene = {}--先清空，防止上一个指令为喂狗指令
    insert(pack.pack(">bbbb",0x03,high,low,num),delay)
    log.info("uartPluse.send","pluse",high,low,num)
    return delay
end

--[[
函数名：read
功能  ：读取串口接收到的数据
参数  ：无
返回值：无
]]
local function read()
    local data = ""
    --底层core中，串口收到数据时：
    --如果接收缓冲区为空，则会以中断方式通知Lua脚本收到了新数据；
    --如果接收缓冲器不为空，则不会通知Lua脚本
    --所以Lua脚本中收到中断读串口数据时，每次都要把接收缓冲区中的数据全部读出，这样才能保证底层core中的新数据中断上来，此read函数中的while语句中就保证了这一点
    while true do
        data = uart.read(UART_ID,"*l")
        if not data or string.len(data) == 0 then break end
        log.info("uartPluse.read hex",data:toHex())
        if data:byte() == 0x03 then
            sys.publish("PLUSE_SENT")
            sys.publish("PLUSE_SENT_UART")
        end
    end
end

--[[
函数名：write
功能  ：通过串口发送数据
参数  ：
        s：要发送的数据
返回值：无
]]
function write(s)
    log.info("uartPluse.write",s:toHex())
    uart.write(UART_ID,s)
end

local function writeOk()
    log.info("uartPluse.writeOk")
end


--保持系统处于唤醒状态，此处只是为了测试需要，所以此模块没有地方调用pm.sleep("testUart")休眠，不会进入低功耗休眠状态
--在开发“要求功耗低”的项目时，一定要想办法保证pm.wake("testUart")后，在不需要串口时调用pm.sleep("testUart")
-- pm.wake("uartPluse")
--注册串口的数据接收函数，串口收到数据后，会以中断方式，调用read接口读取数据
uart.on(UART_ID,"receive",read)
--注册串口的数据发送通知函数
uart.on(UART_ID,"sent",writeOk)

--配置并且打开串口
uart.setup(UART_ID,2400,8,uart.PAR_NONE,uart.STOP_1)
--30秒一次喂狗
if _G.moduleType == 2 then--2g模组需要喂狗
    sys.timerLoopStart(insert,30000,feed)
end

